@@ -1,6 +1,6 @@
 package swat;
 
-our $VERSION = '0.1.52';
+our $VERSION = '0.1.53';
 
 use base 'Exporter'; 
 
@@ -18,10 +18,6 @@ use strict;
 use Test::More;
 use Data::Dumper;
 
-our $HTTP_RESPONSE;
-our @CONTEXT = ();
-our @CONTEXT_C = ();
-our $BLOCK_MODE;
  
 our ($project);
 our ($curl_cmd, $content_file);
@@ -31,6 +27,11 @@ our ($is_swat_package);
 $| = 1;
 
 my $context_populated;
+my $http_response;
+my @context = ();
+my @context_local = ();
+my $block_mode;
+my $captures = [];
 
 sub execute_with_retry {
 
@@ -48,13 +49,13 @@ sub execute_with_retry {
 
 sub make_http_request {
 
-    return $HTTP_RESPONSE if defined $HTTP_RESPONSE;
+    return $http_response if defined $http_response;
 
     my $st = execute_with_retry("$curl_cmd > $content_file && test -s $content_file", $try_num);
 
     open F, $content_file or die $!;
-    $HTTP_RESPONSE = '';
-    $HTTP_RESPONSE.= $_ while <F>;
+    $http_response = '';
+    $http_response.= $_ while <F>;
     close F;
 
     diag `head -c $debug_bytes $content_file` if debug_mod2();
@@ -63,7 +64,7 @@ sub make_http_request {
 
     ok(1,"response saved to $content_file");
 
-    return $HTTP_RESPONSE;
+    return $http_response;
 }
 
 sub populate_context {
@@ -73,14 +74,14 @@ sub populate_context {
     my $data = shift;
     my $i = 0;
 
-    @CONTEXT = ();
+    @context = ();
 
     for my $l ( split /\n/, $data ){
         chomp $l;
         $i++;
-        push @CONTEXT, [$l, $i];        
+        push @context, [$l, $i];        
     }
-    @CONTEXT_C = @CONTEXT;
+    @context_local = @context;
     diag("context populated") if debug_mod2();
     $context_populated=1;
 }
@@ -91,6 +92,15 @@ sub hostname {
     return $a;
 }
 
+sub captures {
+
+    $captures;
+}
+
+sub capture {
+    captures()->[0]
+}
+
 sub check_line {
  
     my $pattern = shift;
@@ -99,29 +109,32 @@ sub check_line {
 
     my $status = 0;
 
-    my @chunks;
 
     my @context_new = ();
+    $captures = [];
 
     populate_context( make_http_request() );
 
     diag("lookup $pattern ...") if debug_mod2();
     if ($check_type eq 'default'){
-        for my $c (@CONTEXT_C){
+        for my $c (@context_local){
             my $ln = $c->[0]; my $next_i = $c->[1];
             if ( index($ln,$pattern) != -1){
                 $status = 1;
-                push @context_new, $CONTEXT[$next_i];
+                push @context_new, $context[$next_i];
             }
         }
     }elsif($check_type eq 'regexp'){
-        for my $c (@CONTEXT_C){
+        for my $c (@context_local){
             my $re = qr/$pattern/;
             my $ln = $c->[0]; my $next_i = $c->[1];
-            if ($ln =~ $re ){
-                push @chunks, $1||$&;
+
+            my @foo = ($ln =~ /$re/g);
+
+            if (scalar @foo){
+                push @{$captures}, [@foo];
                 $status = 1;
-                push @context_new, $CONTEXT[$next_i];
+                push @context_new, $context[$next_i];
             }
         }
     }else {
@@ -131,12 +144,19 @@ sub check_line {
     ok($status,$message);
 
 
-    for my $c (@chunks){
-        diag("line found: $c") if debug_mod1() or debug_mod2();
+    if (debug_mod1() or debug_mod2()){
+        my $k=0;
+        for my $ce (@{$captures}){
+            $k++;
+            diag "captured item N $k";
+            for  my $c (@{$ce}){
+                diag("\tcaptures: $c");
+            }
+        }
     }
 
-    if ($BLOCK_MODE){
-        @CONTEXT_C = @context_new; 
+    if ($block_mode){
+        @context_local = @context_new; 
     }
 
     return
@@ -188,11 +208,11 @@ sub generate_asserts {
 
         if ($l=~ /^\s*begin:\s*$/) { # begin: block marker
             diag("begin: block") if debug_mod2();
-            $BLOCK_MODE=1;
+            $block_mode=1;
             next ENTRY;
         }
         if ($l=~ /^\s*end:\s*$/) { # end: block marker
-            $BLOCK_MODE=0;
+            $block_mode=0;
             populate_context( make_http_request() );
             diag("end: block") if debug_mod2();
             $context_populated=0; # flush current context
@@ -292,7 +312,7 @@ sub handle_generator {
 sub handle_regexp {
 
     my $re = shift;
-    my $message = $BLOCK_MODE ? "$http_meth $path matches | $re" : "$http_meth $path matches $re";
+    my $message = $block_mode ? "$http_meth $path matches | $re" : "$http_meth $path matches $re";
     check_line($re, 'regexp', $message);
     diag "handle_regexp OK. $re" if $ENV{'swat_debug'};
     
@@ -301,7 +321,7 @@ sub handle_regexp {
 sub handle_plain {
 
     my $l = shift;
-    my $message = $BLOCK_MODE ? "$http_meth $path returns | $l" : "$http_meth $path returns $l";
+    my $message = $block_mode ? "$http_meth $path returns | $l" : "$http_meth $path returns $l";
     check_line($l, 'default', $message);
     diag "handle_plain OK. $l" if $ENV{'swat_debug'};   
 }
@@ -444,6 +464,10 @@ It will find all the I<directories with get.txt or post.txt or put.txt files ins
     GET hello/
     GET hello/world
 
+It is possible to run a single swat test setting a C<test_file> variable:
+
+    test_file=hello/get.txt swat example/my-app 127.0.0.1
+
 When you are done with routes you need to set swat data.
 
 
@@ -594,7 +618,7 @@ There are a I<lot of possibilities>! Please follow L<Test::More|search.cpan.org/
     HELLO SWAT
 
 
-=head1 Generators
+=head3 Generators
 
 Swat entries generators is the way to I<create new swat entries on the fly>. Technically speaking it's just a perl code which should return an array reference:
 Generators are very close to perl expressions ( generators code is also get evaled ) with major difference:
@@ -648,7 +672,7 @@ What about to validate web application content with sqlite database entries?
 As an example take a loot at examples/swat-generators-sqlite3 project
 
 
-=head1 Multiline expressions
+=head3 Multiline expressions
 
 Sometimes code looks more readable when you split it on separate chunks. When swat parser meets  C<\> symbols it postpone entry execution and
 add next line to buffer. This is repeated till no C<\> found on next. Finally swat execute I<"accumulated"> swat entity.
@@ -681,12 +705,12 @@ Here are some examples:
 
 Multiline expressions are only allowable for perl expressions and generators 
 
-=head1 Generators and Perl Expressions Scope
+=head3 Generators and Perl Expressions Scope
 
 Swat uses I<perl string eval> when process generators and perl expressions code, be aware of this. 
 Follow L<http://perldoc.perl.org/functions/eval.html> to get more on this.
 
-=head1 PERL5LIB
+=head3 PERL5LIB
 
 Swat adds B<$project_root_directory/lib> to PERL5LIB , so this is convenient convenient to place here custom perl modules:
 
@@ -696,6 +720,53 @@ Swat adds B<$project_root_directory/lib> to PERL5LIB , so this is convenient con
 Take a look at examples/swat-generators-with-lib/ for working example
 
 
+=head3 Captures
+
+Captures is a way to I<access> the data captured in an I<last regexp check>. 
+
+Perl expressions and code generators could access captures calling C<captures()> function.
+Captures() returns array reference holding all captures found during I<last regexp check>.
+Here is a couple of exmaples:
+
+    # Place this in swat data file
+    # check if outpiut contains digits
+    # calculate total amount 
+    # it should be > then 10
+
+    regexp: (\d+)
+    code:                               \
+    my $total=0;                        \
+    for my $c (@{captures()}) {         \
+        $total+=$c->[0];                \         
+    }                                   \
+    cmp_ok( $total,'>',10,"total amount is greater than 10" );
+
+
+    # Place this in swat data file
+    # check if output contains lines with date in `date: YYYY-MM-DD` format
+    # check if first date found is yesterday
+
+    regexp: date: (\d\d\d\d)-(\d\d)-(\d\d)
+    code:                               \
+    use DateTime;                       \
+    my $c = captures()->[0];            \
+    my $dt = DateTime->new( year => $c->[0], month => $c->[1], day => $c->[2]  ); \
+    my $yesterday = DateTime->now->subtract( days =>  1 );     \
+    cmp_ok( DateTime->compare($dt, $yesterday),'==',0,"first day found is - $dt and this is a yesterday" );
+
+
+You also may use C<capture()> function to get a I<first element> of captures array:
+
+
+    # Place this in swat data file
+    # check if outpiut contains digits
+    # calculate total amount 
+    # it should be > then 10
+
+    regexp: (\d+)
+    code: cmp_ok( capture()->[0],'>',10,"fisrt number is greater than 10" );
+
+
 =head1 Anatomy of swat 
 
 Once swat runs it goes through some steps to get job done. Here is description of such a steps executed in orders
@@ -1173,7 +1244,7 @@ B<curl_params>
 
 =item *
 
-B<http_meth> - C<GET|POST>
+B<http_meth> - C<GET|POST|HEAD>
 
 =item *
 
@@ -1314,10 +1385,19 @@ Once we uploaded a module to CPAN repository we can use it:
 Check out existed swat packages here - https://github.com/melezhik/swat-packages/
 
 
-=head1 Debugging
+=head1 Misc
+
+=head2 Debugging
 
 set C<swat_debug> environment variable to 1
 
+=head2 Running a single test
+
+It is possible to run a single swat test setting a C<test_file> variable:
+
+C<test_file>=/relative/path/to/swat/data/file . Path should be relative to the project root directory.
+
+
 
 =head1 Examples
 
